!function($, window, document)
{
	"use strict";
	
	$.fn.selectRange = function(start, end) {
		if (end === undefined) {
			end = start;
		}
		
		return this.each(function() {
			if ('selectionStart' in this) {
				this.selectionStart = start;
				this.selectionEnd = end;
			} else if (this.setSelectionRange) {
				this.setSelectionRange(start, end);
			} else if (this.createTextRange) {
				var range = this.createTextRange();
				range.collapse(true);
				range.moveEnd('character', end);
				range.moveStart('character', start);
				range.select();
			}
		});
	};
	
	$.fn.outerHeightSum = function() {
		var sum = 0;
		
		this.each(function() {
			sum += $(this).outerHeight();
		});
		
		return sum;
	};
	
	XF.Chat = XF.Element.newHandler({
		options: {
			host: '',
			port: 2999,
			path: '',
			communication: 'direct',
			limit: 60,
			audio: '',
			postUrl: '/index.php?chat/post',
			writeUrl: '/index.php?chat/writing',
			editUrl: '/index.php?chat/edit',
			deleteUrl: '/index.php?chat/delete',
			reportUrl: '/index.php?chat/report',
			longPollingUrl: '/index.php?chat/long-polling',
			enabledAudio: false,
			permissions: {},
			ignore: []
        },

        typers: {},
		
		init: function() {
			this.$input = this.$target.find('.input.message').on('keydown', $.proxy(this, 'keyDowned'));
			this.$messagesBlock = this.$target.find('.messages');
			this.$messages = this.$messagesBlock.find('.content');
			this.$messageTyping = this.$target.find('.message--typing');
			this.$messageTypingName = this.$messageTyping.find('.typing_name');
			this.$smilieBox = this.$target.find('.button[data-xf-init="smilie-box"]');
			this.$sendButton = this.$target.find('.button.send').click($.proxy(this, 'postMessage'));
			this.$editorBlock = this.$target.find('.editor-block');

			this.messageTemplate = $('.js-rtcMessage').html();

			this.$messagesBlock.css({
				'padding-right': this.$messagesBlock.outerWidth() - this.$messagesBlock[0].clientWidth + 'px'
			});
			
			if (this.options.audio) {
				this._disableSound = (XF.Cookie.get('chat_disable_sound') === 'true');
				this.$audio = $('<audio />').attr('src', this.options.audio);
			}
			
			this.scrollBottom(true);

			this.connection();
			this.events();
		},

		loaded: false,
		editMode: false,
		editMessageId: false,

		postMessage: function() {
			if (this.loaded) {
				return;
			}
			
			var t = this;
			
			this.$input.prop('disabled', true);
			
			this.loaded = XF.ajax(
				'POST', 
				this.options.postUrl,
				{
					message_id: this.editMode ? this.editMessageId : false,
					message: this.$input.val()
				},
				function() {},
				{
					global: false,
					skipDefaultSuccess: true
				}
			).always(function() {
				t.loaded = false;
				t.$input.prop('disabled', false).focus();
				
				if (t.editMode) {
					t.disableEditMode();
				}
				else {
					t.$input.val('').focus();
				}
			});
		},

		/**
		 * ACTIONS
		 */
			
		actionEdit: function(e, messageId) {
			var t = this;
			
			XF.ajax(
				'GET',
				this.options.editUrl,
				{
					message_id: messageId
				},
				function(data) {
					if (data.message) {
						t.enableEditMode(data.message.message_id);
						t.$input.val(data.message.message).focus();
					}
				},
				{
					global: false,
					skipDefaultSuccess: true
				}
			);
		},

		actionDelete: function(e, messageId) {
			XF.ajax(
				'POST',
				this.options.deleteUrl,
				{
					message_id: messageId
				},
				function(){},
				{
					global: false,
					skipDefaultSuccess: true
				}
			);
		},

		actionReport: function(e, messageId) {
			XF.loadOverlay(XF.stringTranslate(decodeURIComponent(this.options.reportUrl), {'{messageId}': messageId}));
		},

		/**
		 * PERMISSIONS
		 */

		canEdit: function(message) {
			if (this.hasPermission('canEdit') && message.user_id == XF.config.userId) 
			{
				return true;
			}

			return this.hasPermission('canEditAny');
		},

		canDelete: function(message) {
			if (this.hasPermission('canDelete') && message.user_id == XF.config.userId) {
				return true;
			}

			return this.hasPermission('canDeleteAny')
		},

		canReport: function() {
			return this.hasPermission('canReport');
		},

		hasPermission: function(permission) {
			return (this.options.permissions.hasOwnProperty(permission) && this.options.permissions[permission]);
		},

		/**
		 * TAGS
		 */

		isTo: function(message) {
			return (message.to_user_id && message.to_user_id == XF.config.userId);
		},

		isPm: function(message) {
			return (message.pm_user_id);
		},

		controls: function(message) {
			return {
				'controls': this.hasControls(message),
				'is_to': this.isTo(message),
				'is_pm': this.isPm(message),

				'can_edit': this.canEdit(message),
				'can_delete': this.canDelete(message),
				'can_report': this.canReport()
			};
		},

		hasControls: function(message) {
			return (this.canEdit(message) || this.canDelete(message) || this.canReport());
		},
		
		/**
		 * EVENT RENDERERS 
		 */

		loadMessage: function(message) {
			if (this.options.ignore.indexOf(message.user_id) !== -1) {
				return;
			}

			$.extend(message, this.controls(message));

			var $html = $($.parseHTML(Mustache.render(this.messageTemplate, message)));
			XF.activate(this.$messages.append($html));

			if (this.typers.hasOwnProperty(message.user_id)) {
				delete this.typers[message.user_id];
				clearTimeout(this.typingTimeouts[message.user_id]);
				this.checkTyping();
			}

			if (message.user_id != XF.config.userId) {
				this.play();
			}
			
			this.limit();
			this.scrollBottom();
		},

		messageEdited: function(data) {
			var $message = this.$messages.find('.message[data-message-id="' + data.message_id + '"]');

			if ($message.length) {
				if (data.to_user_id) {
					if (data.to_user_id == XF.config.userId) {
						if (!$message.hasClass('to-selected')) {
							$message.addClass('to-selected');
						}	
					}
					else if ($message.hasClass('to-selected')) {
						$message.removeClass('to-selected');
					}
				}
				
				XF.activate($message.find('.message-text').html($(data.message)));
			}
		},

		messagesDeleted: function(data) {
			var t = this;

			if (data.message_ids) {
				$.each(data.message_ids, function() {
					t.$messages.find('.message[data-message-id="' + this + '"]').remove();
				});
			}

			if (data.user_id) {
				t.$messages.find('.message[data-user-id="' + data.user_id + '"]').remove();
			}
		},

		chatCleared: function() {
			this.$messages.empty();
		},

		updateIgnore: function(data) {
			this.options.ignore = $.isArray(data) ? data : [];
		},

		typingTimeouts: {},

		userWriting: function(data) {
			var t = this;

			if (data.user_id == XF.config.userId) {
				return;
			}
			
			if (this.typingTimeouts.hasOwnProperty(data.user_id)) {
				clearTimeout(this.typingTimeouts[data.user_id]);
			}

			this.typers[data.user_id] = data.username;
			this.checkTyping();
			
			this.typingTimeouts[data.user_id] = setTimeout(function() {
				delete t.typers[data.user_id];
				t.checkTyping();
			}.bind(this), 7500);
		},

		checkTyping: function() {
			var userIds = Object.keys(this.typers);

			if (!userIds.length) {
				if (this.$messageTyping.hasClass('message--typing_visible')) {
					this.$messageTyping.removeClass('message--typing_visible');
					this.$messageTypingName.text('');
				}
				return;
			}

			if (userIds.length == 1) {
				this.$messageTypingName.text(XF.phrase('rtc_x_writing_message', {
					'{username}': this.typers[userIds[0]]
				}));
			}

			if (userIds.length == 2) {
				this.$messageTypingName.text(XF.phrase('rtc_x_and_y_writing_message', {
					'{username1}': this.typers[userIds[0]],
					'{username2}': this.typers[userIds[1]]
				}));
			}

			if (userIds.length > 2) {
				this.$messageTypingName.text(XF.phrase('rtc_x_y_and_z_more_writing_message', {
					'{username1}': this.typers[userIds[0]],
					'{username2}': this.typers[userIds[1]],
					'{count}': userIds.length - 2
				}));
			}

			if (!this.$messageTyping.hasClass('message--typing_visible')) {
				this.$messageTyping.addClass('message--typing_visible');
			}
		},

		/**
		 * EVENTS
		 */
		
		events: function() {
			var _this = this;

			this.eventRenders = {
				'new message': function(message) {
					_this.loadMessage(message);
				},

				'edit message': function(data) {
					_this.messageEdited(data);
				},

				'ban': function() {
					window.location.reload(true);
				},

				'delete message': function(data) {
					_this.messagesDeleted(data);
				},

				'user writing': function(data) {
					_this.userWriting(data);
				},

				'clear': function(data) {
					_this.chatCleared();
				},

				'update ignore': function(data) {
					_this.updateIgnore(data);
				}
			};

			if (this.$socket) {
				$.each(this.eventRenders, function (eventKey, func) {
					_this.$socket.on(eventKey, func);
				});
			}
			
			$(document).on('chat:insert', $.proxy(this, 'insertToInput'));
			$(document).on('chat:first-replace', $.proxy(this, 'firstReplace'));
			$(document).on('chat:action-edit', $.proxy(this, 'actionEdit'));
			$(document).on('chat:action-report', $.proxy(this, 'actionReport'));
			$(document).on('chat:action-delete', $.proxy(this, 'actionDelete'));
			$(document).on('chat:disable-sound', $.proxy(this, 'disableSound'));
		},
		
		connection: function() {
            let host = window.location.protocol + '//' + (this.options.host ? this.options.host : window.location.hostname),
            	secure = (window.location.protocol == 'https:');

            switch (this.options.communication)
            {
				case 'long_polling':
				default:
					this.startPolling();
					break;

            	case 'direct':
            		this.$socket = io(host + ':' + this.options.port, {
		                transports: ['websocket'],
		                secure: secure
	            	});
            		break;

            	case 'proxying':
	            	this.$socket = io(host, {
		                transports: ['websocket'],
		                secure: secure,
		                path: this.options.path
		            });
            		break;
            }
        },

		lastEventTime: Date.now() / 1000 | 0,

		startPolling: function () {
			var _this = this;

			XF.ajax('POST',
				this.options.longPollingUrl,
				{
					last_event_time: this.lastEventTime
				},
				function (data) {
					if (data.events) {
						$.each(data.events, function () {
							if (_this.eventRenders.hasOwnProperty(this.event_type)) {
								_this.eventRenders[this.event_type](this.event_data);
							}
						});

						_this.lastEventTime = data.last_event_time;
					}

					_this.startPolling();
				},
				{
					global: false
				}
			);
		},

        /**
         * TRIGGERS
         */

        firstReplace: function(e, insert, replace, regex) {
			var val = this.$input.val();
			
			if (val.indexOf(insert) === 0) {
				val = val.replace(new RegExp(insert), replace);
			}
			else if (val.indexOf(replace) === 0) {
				val = val.replace(new RegExp(replace), insert);
			}
			else if (regex && val.match(regex)) {
				val = val.replace(regex, insert);
			}
			else {
				val = val.length > 0 ? insert + ' ' + val : insert;
			}
			
			if (val.indexOf('{r}') !== -1) {
				this.$input.val(val.replace('{r}', '')).focus().selectRange(val.indexOf('{r}'));
			}
			else {
				this.$input.val(val).focus();
			}
		},
		
		insertToInput: function(e, insert) {
			var val = this.$input.val();
			val = val.length > 0 ? val + ' ' + insert : insert;
			
			this.$input.val(val).focus();
			
			if (val.indexOf('{r}')) {
				this.$input.val(val.replace('{r}', '')).focus().selectRange(val.indexOf('{r}'));
			}
			else {
				this.$input.val(val).focus();
			}
		},

		keyLast: 0,
		
		keyDowned: function(e) {
			var self = this;
			
			if (e.which == 9 || e.keyCode == 9) {
				e.preventDefault();
				this.$smilieBox.trigger('open');
			}
			
			if ((e.which == 27 || e.keyCode == 27) && this.editMode) {
				e.preventDefault();
				this.disableEditMode();
				
				return false;
			}
			
            if (e.which == 13 || e.keyCode == 13) {
                e.preventDefault();
                this.postMessage();

                return false;
            }
			
			if (Date.now() - (this.keyLast || 0) > 1200 && !this.editMode) {
				XF.ajax(
					'POST',
					self.options.writeUrl,
					{},
					function(){},
					{
						global: false,
						skipDefaultSuccess: true
					}
				);
				
				this.keyLast = Date.now();
			}
			
            return true;
        },

		scrollBottom: function(ignoreScroll) {
			if (this.$messagesBlock.length 
				&& ((this.$messagesBlock[0].scrollHeight - this.$messagesBlock.scrollTop() - this.$messagesBlock.height()) < this.$messages.find('.message').slice(-2).outerHeightSum() || ignoreScroll)) 
			{
				if (this.$messagesBlock.find('iframe').length) {
					var t = this;
					this.$messagesBlock.find('iframe').ready(function() 
					{
						t.$messagesBlock.scrollTop(t.$messagesBlock[0].scrollHeight);
					});
				}
				else 
				{
					this.$messagesBlock.scrollTop(this.$messagesBlock[0].scrollHeight);
				}
			}
		},
		
		disableSound: function() {
			this._disableSound = !this._disableSound;
		},
		
		play: function() {
			if (this.$audio && !this._disableSound && this.options.enabledAudio) {
				this.$audio[0].play();
			}
		},

		emitSocket: function(name, value) {
            this.$socket.emit(name, value);
        },
		
		limit: function() {
			var $messages = this.$messages.find('.message');
			
			if ($messages.length > this.options.limit) {
				var countRemove = $messages.length - this.options.limit;
				
				$messages.each(function(index, message) {
					$(message).remove();
					
					if (index + 1 >= countRemove) {
						return false;
					}
				});
			}
		},
		
		enableEditMode: function(messageId) {
			if (!this.editMode) {
				this.oldInputValue = this.$input.val();
			}
			this.editMessageId = messageId;
			this.editMode = true;
		},
		
		disableEditMode: function() {
			this.$input.val(this.oldInputValue).focus();
			this.oldInputValue = '';
			this.editMessageId = 0;
			this.editMode = false;
		}
	});
	
	XF.ChatInsert = XF.Click.newHandler({
		eventNameSpace: 'XFChatInsert',
		
		options: {
			insert: '',
			isReplacer: false,
			replace: '',
			regex: ''
		},

        init: function() {},

        click: function(e) {
			e.preventDefault();

            $(document).trigger(this.getTriggerName(), this.getTriggerParams());
        },
		
		getTriggerName: function() {
			if (this.options.isReplacer) {
				return 'chat:first-replace';
			}
			else {
				return 'chat:insert';
			}
		},
		
		getTriggerParams: function() {
			if (this.options.isReplacer) {
				return [this.options.insert, this.options.replace, new RegExp(this.options.regex)];
			}
			else {
				return [this.options.insert];
			}
		}
	});
	
	XF.ChatMessageAction = XF.Click.newHandler({
		eventNameSpace: 'XFChatAction',

        options: {
        	action: ''
        },

        init: function() {},

        click: function(e) {
			e.preventDefault();
			
            $(document).trigger('chat:action-' + this.options.action, [this.$target.closest('.message').data('messageId')])
        }
	});
	
	XF.ChatDisableSound = XF.Element.newHandler({
		options: {},

        init: function() {
			if (XF.Cookie.get('chat_disable_sound') === 'true') {
				this.$target.find('.fa-volume-up').removeClass('fa-volume-up').addClass('fa-volume-off');
			}
			
			this.$target.click($.proxy(this, 'disable'));
		},

        disable: function(e) {
			e.preventDefault();
			
			XF.Cookie.set('chat_disable_sound', !(XF.Cookie.get('chat_disable_sound') === 'true'));
			
			if (XF.Cookie.get('chat_disable_sound') === 'true') {
				this.$target.find('.fa-volume-up').removeClass('fa-volume-up').addClass('fa-volume-off');
			}
			else {
				this.$target.find('.fa-volume-off').removeClass('fa-volume-off').addClass('fa-volume-up');
			}
			
            $(document).trigger('chat:disable-sound');
        }
	});

	XF.WindowPopup = XF.Click.newHandler({
		eventNameSpace: 'XFWindowPopup',

        options: {
        	name: '',
        	left: 0,
        	top: 0,
        	width: 600,
        	height: 400,
        	autoLeftTop: false
        },

        init: function() {
        	if (this.options.autoLeftTop) {
        		this.options.left = (screen.width - this.options.width) / 2,
				this.options.top = (screen.height - this.options.height) / 2;
        	}
        },

        click: function(e) {
			e.preventDefault();

			window.open(this.$target.attr('href'), this.options.name,
				'toolbar=no,location=no,status=no,menubar=no,scrollbars=yes,resizable=yes'
				+ ',width=' + this.options.width + ',height=' + this.options.height
				+ ',left=' + this.options.left + ',top=' + this.options.top
			);
        }
	});
	
	XF.Element.register('chat', 'XF.Chat');
	XF.Element.register('chat-disable-sound', 'XF.ChatDisableSound');
	
	XF.Click.register('chat-insert', 'XF.ChatInsert');
	XF.Click.register('chat-reload', 'XF.ChatReload');
	XF.Click.register('chat-message-action', 'XF.ChatMessageAction');
	XF.Click.register('window-popup', 'XF.WindowPopup');
}
(window.jQuery, window, document);