!function($, window, document)
{
	"use strict";

	XF.SmilieBox = XF.Element.newHandler({
		options: {
			inputSelector: ''
		},

		loaded: false,

		init: function() {
			var t = this;

			this.$input = $(this.options.inputSelector).on('keydown', function(e) {
				if (e.which == 9 || e.keyCode == 9) {
					e.preventDefault();
					t.$target.click();
					t.$input.focus();
				}
			});
			this.$menu = $($.parseHTML(Mustache.render($('.js-xfSmilieMenu').first().html())));
			
			this.$menu.insertAfter(this.$target);

			this.$target.attr('data-xf-click', 'menu');

			XF.activate(this.$target.parent());

			var handler = XF.Event.getElementHandler(this.$target, 'menu', 'click'),
				scrollTop = 0;

			this.$menu.on('menu:complete', function()
			{
				t.$menuScroll = t.$menu.find('.menu-scroller');

				if (!t.loaded)
				{
					t.loaded = true;

					if (window.IntersectionObserver)
					{
						var observer = new IntersectionObserver(XF.proxy(t, 'onEmojiIntersection'), {
							root: t.$menuScroll[0],
							rootMargin: '0px 0px 100px 0px'
						});
						t.$menuScroll.find('span.smilie--lazyLoad').each(function()
						{
							observer.observe(this);
						});
					}
					else
					{
						t.$menuScroll.onPassive('scroll', loadVisibleImages);
					}

					t.$menuScroll.find('.js-emoji').on('click', XF.proxy(t, 'insertEmoji'));

					var $emojiSearch = t.$menu.find('.js-emojiSearch');
					$emojiSearch.on('input', XF.proxy(t, 'performSearch'));

					t.$menu.find('.js-emojiCloser').on('click', function()
					{
						t.$input.focus();
					});

					t.$menu.on('menu:closed', function()
					{
						scrollTop = t.$menuScroll.scrollTop();
						setTimeout(XF.proxy(t, 'updateRecentEmoji'), 250);
					});
				}

				t.$menuScroll.scrollTop(scrollTop);

				if (!window.IntersectionObserver)
				{
					t.loadVisibleImages(t.$menuScroll);
				}
			});
		},

		insertEmoji: function(e)
		{
			var $target = $(e.currentTarget),
				html = $target.html(),
				$html = $(html),
				val = this.$input.val();

			if ($html.hasClass('smilie--lazyLoad'))
			{
				return;
			}

			this.$input.val(((val.substr(-1) != ' ' && val.length > 0) ? val + ' ' : val) + $target.data('shortname') + ' ');

			setTimeout(function()
			{
				// if an emoji/smilie is selected from the recent list then the inserted
				// emoji moves to the first position. this causes the menu to close when
				// the inserted emoji moves and the recent list is rebuilt.
				// so push the logging out to run separately to everything else.
				XF.logRecentEmojiUsage($target.data('shortname'));
			}, 0);
		},

		loadVisibleImages: function($rowOrEvent)
		{
			var $row = $rowOrEvent,
				t = this;

			if ($rowOrEvent instanceof Event)
			{
				$row = $($rowOrEvent.currentTarget);
			}

			if (!$row.is(':visible'))
			{
				return;
			}

			var visibleRect = $row[0].getBoundingClientRect(),
				visibleBottom = visibleRect.bottom + 100; // 100px offset for visible detection
			$row.children().each(function()
			{
				var $child = $(this),
					childRect = this.getBoundingClientRect();

				if (childRect.bottom < visibleRect.top)
				{
					// area is above what's visible
					return;
				}
				if (childRect.top > visibleBottom)
				{
					// area is below what's visible, so assume everything else is
					return false;
				}

				// otherwise we're visible, so look for smilies here
				$child.find('span.smilie--lazyLoad').each(function()
				{
					var $toLoad = $(this),
						smilieRect = this.getBoundingClientRect();

					if (smilieRect.top <= visibleBottom)
					{
						// smilie is before the end of the visible area, so load
						t.lazyLoadEmoji($toLoad);
					}
				});
			});
		},

		lazyLoadEmoji: function($toLoad)
		{
			var $image = $('<img />').attr({
				'class': $toLoad.attr('class').replace(/(\s|^)smilie--lazyLoad(\s|$)/, ' '),
				alt: $toLoad.attr('data-alt'),
				title: $toLoad.attr('title'),
				src: $toLoad.attr('data-src'),
				'data-shortname': $toLoad.attr('data-shortname')
			});

			var replace = function()
			{
				var f = function()
				{
					$toLoad.replaceWith($image);
				};

				if (window.requestAnimationFrame)
				{
					window.requestAnimationFrame(f);
				}
				else
				{
					f();
				}
			};

			if (!$image.prop('complete'))
			{
				$image.on('load', replace);
			}
			else
			{
				replace();
			}
		},

		updateRecentEmoji: function()
		{
			var recent = XF.getRecentEmojiUsage(),
				$recentHeader = this.$menuScroll.find('.js-recentHeader'),
				$recentBlock = this.$menuScroll.find('.js-recentBlock'),
				$recentList = $recentBlock.find('.js-recentList'),
				$emojiLists = this.$menuScroll.find('.js-emojiList'),
				t = this;

			if (!recent)
			{
				return;
			}

			var $newList = $recentList.clone(),
				newListArr = [];

			$newList.empty();

			for (var i in recent)
			{
				var shortname = recent[i],
					$emoji;

				$emojiLists.each(function()
				{
					var $list = $(this),
						$original = $list.find('.js-emoji[data-shortname="' + shortname + '"]').closest('li');
						$emoji = $original.clone();

					if ($emoji.length)
					{
						$emoji.find('.js-emoji').on('click', XF.proxy(t, 'insertEmoji'));
						newListArr.push($emoji);
						return false;
					}
				});
			}

			for (i in newListArr)
			{
				var $li = newListArr[i];
				$li.appendTo($newList);
			}

			$recentList.replaceWith($newList);

			if ($recentBlock.hasClass('is-hidden'))
			{
				$recentBlock.hide();
				$recentBlock.removeClass('is-hidden');
				$recentHeader.removeClass('is-hidden');
				$recentBlock.xfFadeDown(XF.config.speed.fast);
			}

			this.loadVisibleImages($newList);
		},

		onEmojiIntersection: function(changes, observer)
		{
			var entry, $target;

			for (var i = 0; i < changes.length; i++)
			{
				entry = changes[i];
				if (!entry.isIntersecting)
				{
					continue;
				}

				$target = $(entry.target);
				this.lazyLoadEmoji($target);
				observer.unobserve(entry.target);
			}
		},

		timer: null,

		performSearch: function(e)
		{
			var $input = $(e.currentTarget),
				$fullList = this.$menu.find('.js-emojiFullList'),
				$searchResults = this.$menu.find('.js-emojiSearchResults'),
				t = this;

			clearTimeout(this.timer);

			this.timer = setTimeout(function()
			{
				var value = $input.val();

				if (!value || value.length < 2)
				{
					$searchResults.hide();
					$fullList.show();
					t.loadVisibleImages($fullList);
					return;
				}

				var url = XF.canonicalizeUrl('index.php?editor/smilies-emoji/search');
				XF.ajax('GET', url, {'q': value}, function(data)
				{
					if (!data.html)
					{
						return;
					}

					XF.setupHtmlInsert(data.html, function($html)
					{
						$html.find('.js-emoji').on('click', XF.proxy(t, 'insertEmoji'));

						$fullList.hide();
						$searchResults.replaceWith($html);
					});
				});
			}, 300);
		}
	});

	XF.Element.register('smilie-box', 'XF.SmilieBox');
}
(window.jQuery, window, document);