<?php

namespace BS\RealTimeChat\AddOn\DataType;

use XF\AddOn\DataType\AbstractDataType;

class Command extends AbstractDataType
{
    public function getShortName()
    {
        return 'BS\RealTimeChat:Command';
    }

    public function getContainerTag()
    {
        return 'bs_rtc_commands';
    }

    public function getChildTag()
    {
        return 'bs_rtc_command';
    }

    public function exportAddOnData($addOnId, \DOMElement $container)
    {
        $entries = $this->finder($this->getShortName())
            ->where('addon_id', $addOnId)
            ->order(['command_id'])->fetch();

        foreach ($entries AS $entry)
        {
            $node = $container->ownerDocument->createElement($this->getChildTag());

            $this->exportMappedAttributes($node, $entry);

            $container->appendChild($node);
        }

        return $entries->count() ? true : false;
    }

    public function importAddOnData($addOnId, \SimpleXMLElement $container, $start = 0, $maxRunTime = 0)
    {
        $startTime = microtime(true);

        $entries = $this->getEntries($container, $start);
        if (!$entries)
        {
            return false;
        }

        $conditions = [];
        $existing = [];
        foreach ($entries AS $entry)
        {
            $conditions[] = [
                'command_id' => (string)$entry['command_id']
            ];
        }
        if ($conditions)
        {
            foreach ($this->finder($this->getShortName())->whereOr($conditions)->fetch() AS $command)
            {
                $existing[$command->command_id] = $command;
            }
        }

        $i = 0;
        $last = 0;
        foreach ($entries AS $entry)
        {
            $i++;

            if ($i <= $start)
            {
                continue;
            }

            $commandId = (string)$entry['command_id'];
            $entity = $existing[$commandId] ?? $this->create();

            $entity->getBehavior('XF:DevOutputWritable')->setOption('write_dev_output', false);

            $this->importMappedAttributes($entry, $entity);
            $entity->addon_id = $addOnId;
            $entity->save(true, false);

            if ($this->resume($maxRunTime, $startTime))
            {
                $last = $i;
                break;
            }
        }
        return ($last ?: false);
    }

    public function deleteOrphanedAddOnData($addOnId, \SimpleXMLElement $container)
    {
        $existing = [];
        foreach ($this->findAllForType($addOnId)->fetch() AS $commandId => $command)
        {
            $existing[$commandId] = $command;
        }
        if (!$existing)
        {
            return;
        }

        $entries = $this->getEntries($container) ?: [];

        foreach ($entries AS $entry)
        {
            $commandId = (string)$entry['command_id'];

            if (isset($existing[$commandId]))
            {
                unset($existing[$commandId]);
            }
        }

        array_walk_recursive($existing, function($entity)
        {
            if ($entity instanceof \BS\RealTimeChat\Entity\Command)
            {
                $entity->getBehavior('XF:DevOutputWritable')->setOption('write_dev_output', false);
                $entity->delete();
            }
        });
    }

    public function rebuildActiveChange(\XF\Entity\AddOn $addOn, array &$jobList)
    {
        \XF::runOnce('rebuildActiveBsRtcCommandsCache', function()
        {
            $repo = $this->em->getRepository('BS\RealTimeChat:Command');
            $repo->rebuildCommandsCache();
        });
    }

    protected function getMappedAttributes()
    {
        return [
            'command_id',
            'command_class',
            'command_regex',
            'active'
        ];
    }
}