<?php

namespace BS\RealTimeChat\ChatCommand;

use XF\App;
use BS\RealTimeChat\Entity\Message;

abstract class AbstractCommand
{
    /**
     * @var App
     */
    protected $app;

    /**
     * @var Message
     */
    protected $message;

    public function __construct(App $app, Message $message)
    {
        $this->app = $app;
        $this->message = $message;
    }

    public function render()
    {
        if ($this->canUse())
        {
            return $this->_render();
        }

        return false;
    }

    public function setMessage(Message $message)
    {
        $this->message = $message;
    }

    /**
     * @return Message
     */
    public function getMessage()
    {
        return $this->message;
    }

    /**
     * @return App
     */
    public function app()
    {
        return $this->app;
    }

    /**
     * @return \XF\Db\AbstractAdapter
     */
    public function db()
    {
        return $this->app->db();
    }

    /**
     * @return \XF\Mvc\Entity\Manager
     */
    public function em()
    {
        return $this->app->em();
    }

    /**
     * @param string $repository
     *
     * @return \XF\Mvc\Entity\Repository
     */
    public function repository($repository)
    {
        return $this->app->repository($repository);
    }

    /**
     * @param $finder
     *
     * @return \XF\Mvc\Entity\Finder
     */
    public function finder($finder)
    {
        return $this->app->finder($finder);
    }

    /**
     * @return \XF\DataRegistry
     */
    public function registry()
    {
        return $this->app->registry();
    }

    /**
     * @return \ArrayObject
     */
    public function options()
    {
        return $this->app->options();
    }

    abstract protected function _render();

    protected function canUse()
    {
        return true;
    }
}