<?php

namespace BS\RealTimeChat\Entity;

use XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null banned_id
 * @property int user_id
 * @property int banned_user_id
 * @property int date
 * @property int unban_date
 * @property string reason
 *
 * GETTERS
 * @property string unbanDate
 * @property string reason_crop
 *
 * RELATIONS
 * @property User User
 * @property User BanUser
 */

class Banned extends Entity
{
    public function hasReason()
    {
        return !empty($this->reason);
    }

    public function getUnbanDate()
    {
        return $this->unban_date > 0 ? \XF::language()->dateTime($this->unban_date) : \XF::phrase('never');
    }

    public function getReasonCrop()
    {
        return mb_strlen($this->reason) > 50 ? rtrim(mb_substr($this->reason, 0, 50, 'UTF-8'), "!,.- \n") . '...' : $this->reason;
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_bs_chat_banned';
        $structure->shortName = 'BS\RealTimeChat:Banned';
        $structure->primaryKey = 'banned_id';
        $structure->columns = [
            'banned_id' => ['type' => self::UINT, 'autoIncrement' => true],
            'user_id' => ['type' => self::UINT, 'required' => true],
            'banned_user_id' => ['type' => self::UINT, 'required' => true, 'default' => \XF::visitor()->user_id],
            'date' => ['type' => self::UINT, 'default' => \XF::$time],
            'unban_date' => ['type' => self::UINT, 'default' => 0],
            'reason' => ['type' => self::STR, 'maxLength' => 300, 'required' => false]
        ];

        $structure->getters = [
            'unbanDate' => false,
            'reason_crop' => false
        ];

        $structure->relations = [
            'User' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => 'user_id'
            ],
            'BanUser' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => [['user_id', '=', '$banned_user_id']]
            ]
        ];

        return $structure;
    }
}