<?php

namespace BS\RealTimeChat\LongPoll;

use BS\RealTimeChat\LongPoll\Room\Db;

class Communicator
{
    /** @var \XF\App */
    protected $app;

    /** @var RoomAdapterInterface */
    protected $roomAdapter;

    protected $maxRequestDuration = 25;

    public function __construct(\XF\App $app)
    {
        $this->app = $app;

        $this->setupRoomAdapter();
        $this->setupDefaults();
    }

    protected function setupDefaults()
    {
        $this->setMaxRequestDuration($this->app->options()->realTimeChatClientCommunication['long_polling_limit']);
        $this->roomAdapter->join(\XF::visitor()->user_id);
    }

    public function listen($lastEventTime, $eventTypes = [])
    {
        $pollingTime = 0;

        while ($this->maxRequestDuration >= $pollingTime)
        {
            $events = $this->roomAdapter->hasNewEvents($lastEventTime, $eventTypes);

            if ($events->count())
            {
                return $events;
            }

            sleep(1);
            $pollingTime++;
        }
    }

    public function to($roomId) : RoomAdapterInterface
    {
        return $this->roomAdapter->to($roomId);
    }

    public function in($roomId) : RoomAdapterInterface
    {
        return $this->to($roomId);
    }

    public function emit() : RoomAdapterInterface
    {
        return call_user_func([$this->roomAdapter, 'emit'], ...func_get_args());
    }

    public function join($roomIds) : RoomAdapterInterface
    {
        return $this->roomAdapter->join($roomIds);
    }

    protected function setMaxRequestDuration($duration)
    {
        $this->maxRequestDuration = max(1, $duration);
        set_time_limit($duration + 5);
    }

    protected function setupRoomAdapter()
    {
        $chatConfig = $this->app->config('rtc');
        $adapterClass = $chatConfig['longPollingRoomAdapterClass'] ?? Db::CLASS;
        $this->roomAdapter = new $adapterClass($this->app);
    }
}