<?php

namespace BS\RealTimeChat\Service\Message;

use BS\RealTimeChat\Entity\Message;
use XF\App;
use XF\Entity\User;
use XF\Service\AbstractService;
use XF\Service\ValidateAndSavableTrait;

class Creator extends AbstractService
{
    use ValidateAndSavableTrait;

    /** @var Message */
    protected $message;

    /** @var User */
    protected $user;

    protected $save = true;

    /** @var Manager */
    protected $messageManager;

    /** @var CommandRenderer */
    protected $commandRenderer;

    protected $autoSpamCheck = true;
    protected $performValidations = true;
    protected $renderCommands = true;

    public function __construct(App $app, User $user)
    {
        parent::__construct($app);

        $this->user = $user;

        $this->setupDefaults();
    }

    protected function setupDefaults()
    {
        $this->message = $this->em()->create('BS\RealTimeChat:Message');
        $this->messageManager = $this->service('BS\RealTimeChat:Message\Manager', $this->message);
        $this->commandRenderer = $this->service('BS\RealTimeChat:Message\CommandRenderer', $this->message);
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function getUser()
    {
        return $this->user;
    }

    public function setAutoSpamCheck($check)
    {
        $this->autoSpamCheck = (bool)$check;
    }

    public function setPerformValidations($perform)
    {
        $this->performValidations = (bool)$perform;
    }

    public function setRenderCommands($render)
    {
        $this->renderCommands = (bool)$render;
    }

    public function getPerformValidations()
    {
        return $this->performValidations;
    }

    public function setIsAutomated()
    {
        $this->setAutoSpamCheck(false);
        $this->setPerformValidations(false);
        $this->setRenderCommands(false);
    }

    public function setMessageContent($message, $format = true)
    {
        return $this->messageManager->setMessage($message, $format, $this->performValidations);
    }

    public function checkForSpam()
    {
        if ($this->user->isSpamCheckRequired())
        {
            $this->messageManager->checkForSpam();
        }
    }

    protected function finalSetup()
    {
        $this->message->message_date = round(microtime(true) * 1000);

        if ($this->autoSpamCheck)
        {
            $this->checkForSpam();
        }

        if ($this->renderCommands)
        {
            $this->save = $this->commandRenderer->render();
        }
    }

    protected function _validate()
    {
        $this->finalSetup();

        $this->message->preSave();
        return $this->message->getErrors();
    }

    protected function _save()
    {
        if ($this->save)
        {
            $message = $this->message;

            $db = $this->db();
            $db->beginTransaction();

            $message->save(true, false);

            $db->commit();

            return $message;
        }
        
        return false;
    }

    public function sendNotifications()
    {
        if ($this->message->exists())
        {
            $notifier = $this->service('BS\RealTimeChat:Message\Notifier', $this->message);
            $notifier->setMentionedUserIds($this->messageManager->getMentionedUserIds());
            $notifier->notifyAndEnqueue(3);
        }  
    }
}