<?php

namespace BS\RealTimeChat\Service\Message;

use BS\RealTimeChat\Entity\Message;
use XF;
use XF\App;
use XF\Service\AbstractService;

class Manager extends AbstractService
{
    /** @var Message */
    protected $message;

    /** @var string|null */
    protected $attachmentHash;

    /** @var XF\Service\Message\Preparer */
    protected $preparer;

    public function __construct(App $app, Message $message)
    {
        parent::__construct($app);
        $this->message = $message;
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function setMessage($message, $format = true, $checkValidity = true)
    {
        $this->preparer = $this->getMessagePreparer($format);
        $this->message->message = $this->preparer->prepare($this->zalgoFix($message), $checkValidity);

        return $this->preparer->pushEntityErrorIfInvalid($this->message);
    }

    protected function getMessagePreparer($format = true)
    {
        $preparer = $this->service('XF:Message\Preparer', 'chat_message', $this->message);
        if (!$format)
        {
            $preparer->disableAllFilters();
        }
        
        $preparer->setConstraint('maxLength', $this->app->options()->realTimeChatMessageMaxLength); 

        return $preparer;
    }

    public function checkForSpam()
    {
        $message = $this->message;

        $user = $message->User;

        $checker = $this->app->spam()->contentChecker();
        $checker->check($user, $message->message, [
            'content_type' => 'chat_message'
        ]);

        $decision = $checker->getFinalDecision();
        switch ($decision)
        {
            case 'moderated':
            case 'denied':
                $checker->logSpamTrigger('chat', null);
                $message->error(XF::phrase('your_content_cannot_be_submitted_try_later'));
                break;
        }
    }

    public function getMentionedUsers($limitPermissions = true)
    {
        if ($limitPermissions)
        {
            return $this->message->User->getAllowedUserMentions($this->preparer->getMentionedUsers());
        }
        else
        {
            return $this->preparer->getMentionedUsers();
        }
    }

    public function getMentionedUserIds($limitPermissions = true)
    {
        return array_keys($this->getMentionedUsers($limitPermissions));
    }

    protected function zalgoFix($message)
    {
        return preg_replace('~(?:[\p{M}]{1})([\p{M}])+?~uis', '', $message);
    }
}