<?php

namespace BS\RealTimeChat\Workerman\Events;

use PHPSocketIO\SocketIO;
use PHPSocketIO\Socket;

abstract class AbstractEvent
{
    protected $name;

    /** @var \XF\App */
    protected $app;

    /** @var SocketIO */
    protected $server;

    abstract protected function render(Socket $socket, ...$args);

    public function __construct(SocketIO $server)
    {
        $this->app = \XF::app();
        $this->server = $server;
        $this->resolve();
    }

    protected function resolve()
    {
        $this->server->on('connection', function (Socket $socket)
        {
            $this->em()->clearEntityCache();

            $this->auth($socket);

            if ($this->name)
            {
                $socket->on($this->name, function () use ($socket)
                {
                    $args = func_get_args();
                    array_unshift($args, $socket);

                    call_user_func_array([$this, 'render'], $args);

                    $this->db()->closeConnection();
                });
            }
            else
            {
                $this->render($socket);
            }

            $this->db()->closeConnection();
        });
    }

    protected function auth(Socket $socket)
    {
        if ($socket->request)
        {
            $app = \XF::app();

            parse_str(str_replace('; ', '&', $socket->handshake['headers']['cookie']), $cookie);

            $class = $app->extendClass('XF\Session\Session');

            /** @var \XF\Session\Session $session */
            $session = new $class($app['session.public.storage'], [
                'cookie' => 'session'
            ]);

            // Cannot get ip when proxying
            $session->start(null, $cookie['xf_session'] ?? null);

            $user = $this->getVisitorFromSession($session);
            \XF::setVisitor($user);
        }
    }

    // Copied from \XF|App because this function is protected
    protected function getVisitorFromSession(\XF\Session\Session $session, array $extraWith = [])
    {
        $userRepo = $this->repository('XF:User');
        $sessionUserId = $session->userId;
        $user = $userRepo->getVisitor($sessionUserId, $extraWith);

        if ($user->user_id && $user->user_id == $sessionUserId)
        {
            $userPasswordDate = $user->Profile ? $user->Profile->password_date : 0;
            if ($session->passwordDate != $userPasswordDate)
            {
                $session->logoutUser();
                $user = $userRepo->getVisitor(0);
            }
        }

        return $user;
    }

    /**
     * @return \XF\Db\AbstractAdapter
     */
    public function db()
    {
        return $this->app->db();
    }

    /**
     * @return \XF\Mvc\Entity\Manager
     */
    public function em()
    {
        return $this->app->em();
    }

    /**
     * @param string $identifier
     *
     * @return Mvc\Entity\Finder
     */
    public function finder($identifier)
    {
        return $this->app->finder($identifier);
    }

    /**
     * @param string $identifier
     *
     * @return Mvc\Entity\Repository
     */
    public function repository($identifier)
    {
        return $this->app->repository($identifier);
    }
}